@@ -22,15 +22,19 @@ module Network.IRC.Base (
   ) where
 
 import Data.Maybe
+import Data.Char
+import Data.ByteString (ByteString)
+import qualified Data.ByteString as BS
+import qualified Data.ByteString.Char8 as B8
 
 -- ---------------------------------------------------------
 -- Data Types
 
-type Parameter  = String
-type ServerName = String
-type UserName   = String
-type RealName   = String
-type Command    = String
+type Parameter  = ByteString
+type ServerName = ByteString
+type UserName   = ByteString
+type RealName   = ByteString
+type Command    = ByteString
 
 
 -- | IRC messages are parsed as:
@@ -47,7 +51,7 @@ data Prefix
   = -- | Server Prefix
     Server ServerName
   | -- | Nickname Prefix
-    NickName String (Maybe UserName) (Maybe ServerName)
+    NickName ByteString (Maybe UserName) (Maybe ServerName)
     deriving (Show,Read,Eq)
 
 
@@ -56,27 +60,32 @@ data Prefix
 
 
 -- | Encode a message to its string representation
-encode :: Message -> String
-encode m = showMessage m
+encode :: Message -> ByteString
+encode = showMessage
 
 -- | This is the deprecated version of encode
-render :: Message -> String
+render :: Message -> ByteString
 render  = encode
 
-showMessage :: Message -> String
-showMessage (Message p c ps) = showMaybe p ++ c ++ showParameters ps
-  where showMaybe = maybe "" ((++ " ") . (':':) . showPrefix)
+showMessage :: Message -> ByteString
+showMessage (Message p c ps) = showMaybe p `BS.append` c `BS.append` showParameters ps
+  where showMaybe Nothing = BS.empty
+        showMaybe (Just prefix) = BS.concat [ B8.pack ":"
+                                            , showPrefix prefix
+                                            , B8.pack " " ]
 
-showPrefix :: Prefix -> String
+bsConsAscii :: Char -> ByteString -> ByteString
+bsConsAscii c = BS.cons (fromIntegral . ord $ c)
+
+showPrefix :: Prefix -> ByteString
 showPrefix (Server s)       = s
-showPrefix (NickName n u h) = n ++ showMaybe '!' u ++ showMaybe '@' h
-  where showMaybe c e = maybe "" (c:) e
+showPrefix (NickName n u h) = BS.concat [n, showMaybe '!' u, showMaybe '@' h]
+  where showMaybe c e = maybe BS.empty (bsConsAscii c) e
 
-showParameters :: [Parameter] -> String
-showParameters []     = []
-showParameters params = " " ++ (unwords $ showp params)
-  where showp [p] | ' ' `elem` p || null p || head p == ':' = [':' : p]
-                  | otherwise = [p]
+showParameters :: [Parameter] -> ByteString
+showParameters []     = BS.empty
+showParameters params = BS.intercalate (B8.pack " ") (BS.empty : showp params)
+  where showp [p]    = [bsConsAscii ':' p]
         showp (p:ps) = p : showp ps
         showp []     = []
 
@@ -86,13 +95,13 @@ showParameters params = " " ++ (unwords $ showp params)
 -- | Translate a reply into its text description.
 --   If no text is available, the argument is returned.
 translateReply :: Command -- ^ Reply
-               -> String  -- ^ Text translation
+               -> ByteString  -- ^ Text translation
 translateReply r = fromMaybe r $ lookup r replyTable
 
 
 -- One big lookup table of codes and errors
-replyTable :: [(String,String)]
-replyTable  =
+replyTable :: [(ByteString, ByteString)]
+replyTable  = map mkPair
   [ ("401","ERR_NOSUCHNICK")
   , ("402","ERR_NOSUCHSERVER")
   , ("403","ERR_NOSUCHCHANNEL")
@@ -212,3 +221,5 @@ replyTable  =
   , ("258","RPL_ADMINLOC2")
   , ("259","RPL_ADMINEMAIL")
   ]
+  where
+  mkPair (a,b) = (B8.pack a, B8.pack b)
